#include "Enemy.h"
#include "DxLib.h"
#include "Player.h"
#include "Effect.h"
#include "Item.h"
#include "Level.h"

std::list<Enemy*> Enemy::_enemyList;
int Enemy::_img[];

Enemy::Sound Enemy::_snd;
const double Enemy::VELOCITY_BASIC( 3.0 );
const double Enemy::VELOCITY_EXPAND( 5.0 );
const double Enemy::VELOCITY_GIANT( 2.5 );
const double Enemy::VELOCITY_STAR( 4.0 );
const double Enemy::VELOCITY_SPIDER( 3.5 );
const double Enemy::TURN_BASIC( 0.1 );
const double Enemy::TURN_SPIDER( 0.1 );
const double Enemy::REACTION_EXPAND( 1.0 );
const double Enemy::MARGIN( 96 );
const double Enemy::ACCELERATION_EXPAND( VELOCITY_EXPAND / 30 );
const double Enemy::VELOCITY_BULLET( 5.0 );

#define SQUARE( num ) ((num)*(num))

int Interpolation( int x, int xBegin, int xEnd, int yBegin, int yEnd ){
	int y;
	if( xBegin < xEnd ){
		if( x < xBegin ) return yBegin;
		if( xEnd < x ) return yEnd;
	}
	else{
		if( xBegin < x ) return yBegin;
		if( x < xEnd ) return yEnd;
	}
	y = yEnd - yBegin;
	y *= x - xBegin;
	y /= xEnd - xBegin;
	y += yBegin;
	return y;
}

Enemy::Enemy( double x, double y, Enemy::Type type, double th ) {
	_x = x;
	_y = y;
	_vx = 0;
	_vy = 0;
	_count = 0;
	_type = type;
	_th = th;
	_exist = true;
	_flash = false;
	_enemyList.push_back( this );
	switch( type ){
	case BASIC:
		_life = LIFE_BASIC;
		return;
	case EXPAND:
		_life = LIFE_EXPAND;
		return;
	case STAR:
		_life = LIFE_STAR;
		return;
	case GIANT:
		_life = LIFE_GIANT;
		return;
	case BULLET:
		_life = 1;
		return;
	case SPIDER:
		_life = LIFE_SPIDER;
		return;
	}
}
Enemy::Enemy( double x, double y, Enemy::Type type, double th, double v ){
	_x = x;
	_y = y;
	double shake = Level::GetRankCoefficient();
	shake -= shake / 2;
	th += ( GetRand( 100 ) * shake ) / 100;
	_vx = v * cos( th );
	_vy = v * sin( th );
	_count = 0;
	_type = type;
	_th = th;
	_exist = true;
	_flash = false;
	_enemyList.push_back( this );
	switch( type ){
	case BASIC:
		_life = LIFE_BASIC;
		return;
	case EXPAND:
		_life = LIFE_EXPAND;
		return;
	case STAR:
		_life = LIFE_STAR;
		return;
	case GIANT:
		_life = LIFE_GIANT;
		return;
	case BULLET:
		_life = 1;
		return;
	case SPIDER:
		_life = LIFE_SPIDER;
		return;
	}
}
void Enemy::Move( void ) {
	double playerX = 0, playerY = 0;
	double x, y, th;
	switch( _type ) {
	case BASIC:
		{
			if( _count == GetCycleBasic() ){
				_count = 0;
			}
			Player::GetPos( playerX, playerY );
			x = playerX - _x;
			y = playerY - _y;
			th = 0;
			if ( x != 0 || y != 0 ) th = atan2( y, x );
			if ( sin( th - _th ) > 0 ) {
				_th += TURN_BASIC;
				if ( sin( th - _th ) < 0 ) {
					_th = th;
				}
			}
			else {
				_th -= TURN_BASIC;
				if ( sin( th - _th ) > 0 ) {
					_th = th;
				}
			}
			_vx = VELOCITY_BASIC * cos( _th );
			_vy = VELOCITY_BASIC * sin( _th );
			if( _count == GetCycleBasic() / 2 ){
				double velocity = GetBulletVelocity();
				new Enemy( _x, _y, Enemy::BULLET, _th - DX_PI / 4, velocity );
				new Enemy( _x, _y, Enemy::BULLET, _th, velocity );
				new Enemy( _x, _y, Enemy::BULLET, _th + DX_PI / 4, velocity );
			}
			_x += _vx;
			_y += _vy;
			break;
		}
	case EXPAND:
		{
			Player::GetPos( playerX, playerY );
			if( _x < playerX ) _vx += ACCELERATION_EXPAND;
			if( playerX < _x ) _vx -= ACCELERATION_EXPAND;
			if( _y < playerY ) _vy += ACCELERATION_EXPAND;
			if( playerY < _y ) _vy -= ACCELERATION_EXPAND;
			if( SQUARE( VELOCITY_EXPAND ) < SQUARE( _vx ) + SQUARE( _vy ) ){
				_th = atan2( _vy, _vx );
				_vx = VELOCITY_EXPAND * cos( _th );
				_vy = VELOCITY_EXPAND * sin( _th );
			}
			_x += _vx;
			_y += _vy;
			break;
		}
	case STAR:
		{
			if( _count == CYCLE_STAR ){
				_count = 0;
			}
			// vC[̕
			if( _count == 0 ){
				Player::GetPos( playerX, playerY );
				x = playerX - _x;
				y = playerY - _y;
				_th = 0;
				if ( x != 0 || y != 0 ) _th = atan2( y, x );
			}
			// ړvZX
			if( _count == 0 ){
				_vx = VELOCITY_STAR * cos( _th );
				_vy = VELOCITY_STAR * sin( _th );
			}
			if( _count == ( CYCLE_STAR * 2 ) / 3 ){
				_vx = 0;
				_vy = 0;
			}
			/*if( _count < ( CYCLE_STAR * 2 ) / 3 ){
				_vx = VELOCITY_STAR * cos( _th );
				_vy = VELOCITY_STAR * sin( _th );
			}*/
			// evZX
			if( _count == ( CYCLE_STAR * 5 ) / 6 ){
				double v = GetBulletVelocity();
				int way = static_cast< int >( ( Level::GetRankCoefficient() + 1 ) * 5 );
				for( int i = 0; i < way; ++i ){
					new Enemy( _x, _y, Enemy::BULLET, _th + i * DX_PI * 2 / way, v );
				}
			}
			_x += _vx;
			_y += _vy;
			break;
		}
	case GIANT:
		{
			_x += VELOCITY_GIANT * cos( _th );
			_y += VELOCITY_GIANT * sin( _th );
			if( _x < -MARGIN || 640 + MARGIN < _x ||
				_y < -MARGIN || 480 + MARGIN < _y )
				_exist = false;
			break;
		}
	case BULLET:
		{
			_x += _vx;
			_y += _vy;
			if( _x < -100 || 740 < _x || _y < -100 || 580 < _y )
				_exist = false;
			break;
		}
	case SPIDER:
		{
			if( _count == CYCLE_SPIDER ){
				_count = 0;
			}
			Player::GetPos( playerX, playerY );
			x = playerX - _x;
			y = playerY - _y;
			th = 0;
			if ( x != 0 || y != 0 ) th = atan2( y, x );
			if ( sin( th - _th ) > 0 ) {
				_th += TURN_SPIDER;
				if ( sin( th - _th ) < 0 ) {
					_th = th;
				}
			}
			else {
				_th -= TURN_SPIDER;
				if ( sin( th - _th ) > 0 ) {
					_th = th;
				}
			}
			_x += VELOCITY_SPIDER * cos( _th );
			_y += VELOCITY_SPIDER * sin( _th );
			int num = static_cast< int >( 5 * ( Level::GetRankCoefficient() + 1 ) );
			if( CYCLE_SPIDER / 2 <= _count && _count % ( CYCLE_SPIDER / num / 2 ) == 0 ){
				double v = GetBulletVelocity();
				new Enemy( _x, _y, Enemy::BULLET, _th, v );
			}
			break;
		}
	}
}
void Enemy::MoveAll( void ) {
	for ( auto it = _enemyList.begin(); it != _enemyList.end(); ) {
		( *it )->Move();
		if ( ( *it )->_exist == FALSE ) {
			delete ( *it );
			it = _enemyList.erase( it );
			continue;
		}
		++it;
	}
}
int Enemy::Draw( void ) {
	int rv = 0;

	int r, g, b;
	SetDrawBlendMode( DX_BLENDMODE_ADD, 255 );
	switch( _type ) {
	case BASIC:
		{
			if( _flash ){
				SetDrawBlendMode( DX_BLENDMODE_INVSRC, 255 );
				rv |= DrawRotaGraph( static_cast< int >( _x ), static_cast< int >( _y ), 1.5, _th, _img[ 0 ], TRUE );
				SetDrawBlendMode( DX_BLENDMODE_ADD, 255 );
				rv |= DrawRotaGraph( static_cast< int >( _x ), static_cast< int >( _y ), 1.5, _th, _img[ 0 ], TRUE );
				SetDrawBlendMode( DX_BLENDMODE_NOBLEND, 0 );
			}
			else{
				r = Interpolation( _life, LIFE_BASIC, LIFE_BASIC / 2, 0, 255 );
				g = Interpolation( _life, LIFE_BASIC / 2, 1, 255, 0 );
				b = Interpolation( _life, LIFE_BASIC, LIFE_BASIC / 2, 255, 0 );
				SetDrawBright( r, g, b );
				rv = DrawRotaGraph( static_cast< int >( _x ), static_cast< int >( _y ), 1.5, _th, _img[ 0 ], TRUE );
				SetDrawBright( 254, 254, 254 );
			}
			break;
		}
	case BULLET:
		{
			rv |= DrawRotaGraph( static_cast< int >( _x ), static_cast< int >( _y ),
				1.0, 0.0, _img[ BULLET ], TRUE );
			SetDrawBright( 254, 254, 254 );
			break;
		}
	case EXPAND:
		{
			int radius = GetRadius();
			if( _flash ){
				SetDrawBlendMode( DX_BLENDMODE_INVSRC, 255 );
				rv |= DrawExtendGraph(
					static_cast< int >( _x ) - radius - 4, static_cast< int >( _y ) - radius - 4,
					static_cast< int >( _x ) + radius + 4, static_cast< int >( _y ) + radius + 4,
					_img[ 1 ], TRUE );
				SetDrawBlendMode( DX_BLENDMODE_ADD, 255 );
				rv |= DrawExtendGraph(
					static_cast< int >( _x ) - radius - 4, static_cast< int >( _y ) - radius - 4,
					static_cast< int >( _x ) + radius + 4, static_cast< int >( _y ) + radius + 4,
					_img[ 1 ], TRUE );
				SetDrawBlendMode( DX_BLENDMODE_NOBLEND, 0 );
			}
			else{
				r = Interpolation( _life, LIFE_EXPAND, LIFE_EXPAND / 2, 0, 255 );
				g = Interpolation( _life, LIFE_EXPAND / 2, 1, 255, 0 );
				b = Interpolation( _life, LIFE_EXPAND, LIFE_EXPAND / 2, 255, 0 );
				SetDrawBright( r, g, b );
				rv |= DrawExtendGraph(
					static_cast< int >( _x ) - radius - 4, static_cast< int >( _y ) - radius - 4,
					static_cast< int >( _x ) + radius + 4, static_cast< int >( _y ) + radius + 4,
					_img[ 1 ], TRUE );
				SetDrawBright( 254, 254, 254 );
			}
			break;
		}
	case STAR:
		{
			int radius = GetRadius();
			if( _flash ){
				SetDrawBlendMode( DX_BLENDMODE_INVSRC, 255 );
				rv |= DrawExtendGraph(
					static_cast< int >( _x ) - radius - 8, static_cast< int >( _y ) - radius - 8,
					static_cast< int >( _x ) + radius + 8, static_cast< int >( _y ) + radius + 8,
					_img[ STAR ], TRUE );
				SetDrawBlendMode( DX_BLENDMODE_ADD, 255 );
				rv |= DrawExtendGraph(
					static_cast< int >( _x ) - radius - 8, static_cast< int >( _y ) - radius - 8,
					static_cast< int >( _x ) + radius + 8, static_cast< int >( _y ) + radius + 8,
					_img[ STAR ], TRUE );
				SetDrawBlendMode( DX_BLENDMODE_NOBLEND, 0 );
			}
			else{
				r = Interpolation( _life, LIFE_STAR, LIFE_STAR / 2, 0, 255 );
				g = Interpolation( _life, LIFE_STAR / 2, 1, 255, 0 );
				b = Interpolation( _life, LIFE_STAR, LIFE_STAR / 2, 255, 0 );
				SetDrawBright( r, g, b );
				rv |= DrawExtendGraph(
					static_cast< int >( _x ) - radius - 8, static_cast< int >( _y ) - radius - 8,
					static_cast< int >( _x ) + radius + 8, static_cast< int >( _y ) + radius + 8,
					_img[ STAR ], TRUE );
				SetDrawBright( 254, 254, 254 );
			}
			break;
		}
	case GIANT:
		{
			int radius = GetRadius();
			radius += 4;
			int size;
			GetGraphSize( _img[ GIANT ], &size, &size );
			if( _flash ){
				SetDrawBlendMode( DX_BLENDMODE_INVSRC, 255 );
				rv |= DrawRotaGraph( static_cast< int >( _x ), static_cast< int >( _y ),
					static_cast< double >( radius ) * 2 / size, _th, _img[ GIANT ], TRUE );
				SetDrawBlendMode( DX_BLENDMODE_ADD, 255 );
				rv |= DrawRotaGraph( static_cast< int >( _x ), static_cast< int >( _y ),
					static_cast< double >( radius ) * 2 / size, _th, _img[ GIANT ], TRUE );
				SetDrawBlendMode( DX_BLENDMODE_NOBLEND, 0 );
			}
			else{
				r = Interpolation( _life, LIFE_GIANT	, LIFE_GIANT / 2, 0		, 255	);
				g = Interpolation( _life, LIFE_GIANT / 2, 1				, 255	, 0		);
				b = Interpolation( _life, LIFE_GIANT	, LIFE_GIANT / 2, 255	, 0		);
				SetDrawBright( r, g, b );
				rv |= DrawRotaGraph( static_cast< int >( _x ), static_cast< int >( _y ),
					static_cast< double >( radius ) * 2 / size, _th, _img[ GIANT ], TRUE );
				SetDrawBright( 254, 254, 254 );
			}
			break;
		}
	case SPIDER:
		{
			if( _flash ){
				SetDrawBlendMode( DX_BLENDMODE_INVSRC, 255 );
				rv |= DrawRotaGraph( static_cast< int >( _x ), static_cast< int >( _y ), 1.5, _th, _img[ SPIDER ], TRUE );
				SetDrawBlendMode( DX_BLENDMODE_ADD, 255 );
				rv |= DrawRotaGraph( static_cast< int >( _x ), static_cast< int >( _y ), 1.5, _th, _img[ SPIDER ], TRUE );
				SetDrawBlendMode( DX_BLENDMODE_NOBLEND, 0 );
			}
			else{
				r = Interpolation( _life, LIFE_SPIDER, LIFE_SPIDER / 2, 0, 255 );
				g = Interpolation( _life, LIFE_SPIDER / 2, 1, 255, 0 );
				b = Interpolation( _life, LIFE_SPIDER, LIFE_SPIDER / 2, 255, 0 );
				SetDrawBright( r, g, b );
				rv = DrawRotaGraph( static_cast< int >( _x ), static_cast< int >( _y ), 1.5, _th, _img[ SPIDER ], TRUE );
				SetDrawBright( 254, 254, 254 );
			}
			break;
		}
	}
	_flash = false;
	SetDrawBlendMode( DX_BLENDMODE_NOBLEND, 0 );
#ifdef _DEBUG
	DrawCircle( static_cast< int >( _x ), static_cast< int >( _y ),
		GetRadius(), GetColor( 255, 0, 0 ), false );
#endif
	_count++;
	return rv;
}
void Enemy::DrawAll( void ) {
	for ( auto it = _enemyList.begin(); it != _enemyList.end();) {
		if ( ( *it )->_exist == FALSE ) {
			delete ( *it );
			it = _enemyList.erase( it );
			continue;
		}
		else {
			if( ( *it )->Draw() ){
				return;
			}
		}
		++it;
	}
}
int Enemy::LoadImg( void ) {
	_img[ BASIC		] = DxLib::LoadGraph( "img\\enemy_00.png"	);
	_img[ EXPAND	] = DxLib::LoadGraph( "img\\enemy_01.png"	);
	_img[ STAR		] = DxLib::LoadGraph( "img\\enemy_02.png"	);
	_img[ GIANT		] = DxLib::LoadGraph( "img\\enemy_03.png"	);
	_img[ SPIDER	] = DxLib::LoadGraph( "img\\spider.png"		);
	_img[ BULLET	] = DxLib::LoadGraph( "img\\bullet.png"		);
	return 0;
}
int Enemy::GetRadius( void ) {
	switch( _type ) {
	case BASIC:
		return 20;
	case BULLET:
		return 4;
	case GIANT:
		return 80;
	case STAR:
		return 32;
	case EXPAND:
		{
			int r = RADIUS_MAX_EXPAND - RADIUS_MIN_EXPAND;
			r *= LIFE_EXPAND - _life;
			r /= LIFE_EXPAND - 1;
			r += RADIUS_MIN_EXPAND;
			return r;
		}
	case SPIDER:
		return 40;
	}
	return 0;
}
void Enemy::Hit( void ){
	_life--;
	_flash = true;
	PlaySoundMem( _snd._damage, DX_PLAYTYPE_BACK );
	if( _life <= 0 ){
		PlaySoundMem( _snd._break, DX_PLAYTYPE_BACK );
		_exist = false;
		if( Item::PROBABILITY * 100 > GetRand( 100 ) ){
			new Item( _x, _y,
				Item::VELOCITY * cos( static_cast< double >( GetRand( 100 ) ) ),
				Item::VELOCITY * sin( static_cast< double >( GetRand( 100 ) ) ) );
		}
		switch( _type ){
		case BASIC:
			AddScore( 100 );
			new Effect( _x, _y, 2 );
			break;
		case GIANT:
			AddScore( 5024 );
			new Effect( _x, _y, 5 );
			break;
		case STAR:
			AddScore( 400 );
			new Effect( _x, _y, 3 );
			break;
		case EXPAND:
			AddScore( 200 );
			new Effect( _x, _y, 1 );
			break;
		case SPIDER:
			AddScore( 800 );
			new Effect( _x, _y, 4 );
			break;
		}
	}
}
void Enemy::HitExpand( Shot& shot ){
	double th = shot.GetAngle();
	_vx += REACTION_EXPAND * cos( th );
	_vy += REACTION_EXPAND * sin( th );
}
int Enemy::LoadSnd( int volume ){
	if( volume > 255 ) volume = 255;
	_snd._damage = LoadSoundMem( "snd\\hit.wav" );
	_snd._break = LoadSoundMem( "snd\\break.wav" );
	ChangeVolumeSoundMem( volume, _snd._damage );
	ChangeVolumeSoundMem( volume, _snd._break );
	return 0;
}
void Enemy::Create( double pos, Type type, Direction direction ){
	double x, y, th;

	static const double SCREEN_TOP = 0;
	static const double SCREEN_BOTTOM = 480;
	static const double SCREEN_LEFT = 0;
	static const double SCREEN_RIGHT = 640;
	static const double ANGLE_RIGHT	= DX_PI * 0.0;
	static const double ANGLE_DOWN	= DX_PI * 0.5;
	static const double ANGLE_LEFT	= DX_PI * 1.0;
	static const double ANGLE_UP	= DX_PI * 1.5;

	switch( direction ){
	case TOP:
		x = pos;
		y = SCREEN_TOP - MARGIN;
		th = ANGLE_DOWN;
		break;
	case BOTTOM:
		x = pos;
		y = SCREEN_BOTTOM + MARGIN;
		th = ANGLE_UP;
		break;
	case LEFT:
		x = SCREEN_LEFT - MARGIN;
		y = pos;
		th = ANGLE_RIGHT;
		break;
	case RIGHT:
		x = SCREEN_RIGHT + MARGIN;
		y = pos;
		th = ANGLE_LEFT;
		break;
	}

	new Enemy( x, y, type, th );
}
double Enemy::GetBulletVelocity( void ){
	return VELOCITY_BULLET + VELOCITY_BULLET * ( GetRand( 100 ) * Level::GetRankCoefficient() ) / 100;
}
int Enemy::GetCycleBasic( void ){
	return static_cast< int >( CYCLE_BASIC / ( Level::GetRankCoefficient() + 1 ) );
}