#ifndef ENEMY_H
#define ENEMY_H

#include <list>
#include "Shot.h"

void AddScore( int num );

class Enemy {
public:
	enum Type {
		BASIC,	// ǂ
		EXPAND,	// c
		STAR,	// UI
		GIANT,	// ł
		BULLET,	// Ge
		SPIDER,	// w
	};
	enum Direction{
		TOP,
		BOTTOM,
		LEFT,
		RIGHT,
	};
	Enemy( double x, double y, Enemy::Type type, double th );
	Enemy( double x, double y, Enemy::Type type, double th, double v );
	void Move( void );
	static void MoveAll( void );
	static void DrawAll( void );
	int Draw( void );
	static int LoadImg( void );
	int GetRadius( void );
	static std::list<Enemy*>& GetEnemyList( void ) {
		return _enemyList;
	}
	void GetPos( double& x, double& y ) {
		x = _x;
		y = _y;
	}
	void Hit();
	void HitExpand( Shot& );
	Type GetType( void ){
		return _type;
	}
	bool IsExist( void ){
		return _exist;
	}
	static void Create( double, Type, Direction );
	static const double MARGIN;
	static double GetBulletVelocity( void );
private:
	double _x;
	double _y;
	double _vx;
	double _vy;
	Type _type;
	bool _exist;
	double _th;
	int _count;
	int _life;
	bool _flash;

	static const int IMG_MAX = 6;
	static int _img[ IMG_MAX ];
	static std::list<Enemy*> _enemyList;

	enum{
		LIFE_BASIC	= 10,
		LIFE_EXPAND = 20,
		LIFE_GIANT	= 80,
		LIFE_STAR	= 30,
		LIFE_SPIDER	= 40,
	};
	static const int RADIUS_MIN_EXPAND = ( 24 - 8 ) / 2;
	static const int RADIUS_MAX_EXPAND = ( 192 - 8 ) / 2;
	static const double VELOCITY_BASIC;
	static const double VELOCITY_EXPAND;
	static const double VELOCITY_GIANT;
	static const double VELOCITY_STAR;
	static const double VELOCITY_SPIDER;
	static const double VELOCITY_BULLET;
	static const double TURN_BASIC;
	static const double TURN_SPIDER;
	static const double ACCELERATION_EXPAND;
	static const double REACTION_EXPAND;
	static const int CYCLE_BASIC = 180;
	static const int CYCLE_STAR = 90;
	static const int CYCLE_SPIDER = 150;
	static struct Sound{
		int _damage;
		int _break;
	}_snd;
public:
	static int LoadSnd( int );
	static void EraseAll( void ){
		for( auto it = _enemyList.begin(); it != _enemyList.end(); ){
			delete ( *it );
			it = _enemyList.erase( it );
		}
	}
	static int GetCycleBasic( void );
};

#endif