#include "DxLib.h"
#include "Input.h"
#include <math.h>
#include <fstream>
#include <string>

Input* Input::_p = NULL;

Input::Input () {
	_p = this;
	for ( int i = 0; i < ANALOG_MAX; ++i ) {
		_analog[ i ] = 0;
	}
	LoadConfig();
	DxLib::SetUseDirectInputFlag( _config._useDirectInputFlag );
}

void Input::Update ( void ) {
	GetJoypadDirectInputState( DX_INPUT_PAD1, &_dInputJoystate );
	DxLib::GetHitKeyStateAll( _key );
	{
#ifdef _DEBUG
		int y = 160;
		DrawFormatString( 0, y += 16, GetColor( 255, 255, 255 ), "X: %d", _dInputJoystate.X );
		DrawFormatString( 0, y += 16, GetColor( 255, 255, 255 ), "Y: %d", _dInputJoystate.Y );
		DrawFormatString( 0, y += 16, GetColor( 255, 255, 255 ), "Z: %d", _dInputJoystate.Z );
		DrawFormatString( 0, y += 16, GetColor( 255, 255, 255 ), "Rx: %d", _dInputJoystate.Rx );
		DrawFormatString( 0, y += 16, GetColor( 255, 255, 255 ), "Ry: %d", _dInputJoystate.Ry );
		DrawFormatString( 0, y += 16, GetColor( 255, 255, 255 ), "Rz: %d", _dInputJoystate.Rz );
#endif
	}
	// AiO͂
	_analog[ 0 ] = _dInputJoystate.X;
	_analog[ 1 ] = _dInputJoystate.Y;
	_analog[ 2 ] = 0;
	_analog[ 3 ] = 0;
	switch( _config._attackAnalog[ 0 ] ){
	case 0:
		_analog[ 2 ] = _dInputJoystate.Z;
		break;
	case 1:
		_analog[ 2 ] = _dInputJoystate.Rx;
		break;
	case 2:
		_analog[ 2 ] = _dInputJoystate.Ry;
		break;
	case 3:
		_analog[ 2 ] = _dInputJoystate.Rz;
		break;
	}
	switch( _config._attackAnalog[ 1 ] ){
	case 0:
		_analog[ 3 ] = _dInputJoystate.Z;
		break;
	case 1:
		_analog[ 3 ] = _dInputJoystate.Rx;
		break;
	case 2:
		_analog[ 3 ] = _dInputJoystate.Ry;
		break;
	case 3:
		_analog[ 3 ] = _dInputJoystate.Rz;
		break;
	}

	// {^͂
	if( _dInputJoystate.Buttons[ _config._attackLeft - 1	] ) _analog[ 2 ] -= 1000;
	if( _dInputJoystate.Buttons[ _config._attackRight - 1	] ) _analog[ 2 ] += 1000;
	if( _dInputJoystate.Buttons[ _config._attackUp - 1		] )	_analog[ 3 ] -= 1000;
	if( _dInputJoystate.Buttons[ _config._attackDown - 1	] )	_analog[ 3 ] += 1000;
	if( _dInputJoystate.Buttons[ _config._pause - 1			] ) _button[ 0 ] += 1;
	else _button[ 0 ] = 0;
	if( _button[ 0 ] > 100 ) _button[ 0 ] = 100;

	// L[͂
	if( _key[ KEY_INPUT_D ] ){
		_analog[ 0 ] += 1000;
	}
	if( _key[ KEY_INPUT_A ] ){
		_analog[ 0 ] -= 1000;
	}
	if( _key[ KEY_INPUT_S ] ){
		_analog[ 1 ] += 1000;
	}
	if( _key[ KEY_INPUT_W ] ){
		_analog[ 1 ] -= 1000;
	}
	if( _key[ KEY_INPUT_RIGHT ] ){
		_analog[ 2 ] += 1000;
	}
	if ( _key[ KEY_INPUT_LEFT ] ) {
		_analog[ 2 ] -= 1000;
	}
	if ( _key[ KEY_INPUT_DOWN ] ) {
		_analog[ 3 ] += 1000;
	}
	if ( _key[ KEY_INPUT_UP ] ) {
		_analog[ 3 ] -= 1000;
	}

	// POV
	if( _dInputJoystate.POV[ 0 ] != -1 && _config._useDirectInputFlag ){
		_analog[ 0 ] += static_cast< int >( 1000 * cos( ( static_cast< int >( _dInputJoystate.POV[ 0 ] ) - 9000 ) * DX_PI / 18000 ) );
		_analog[ 1 ] += static_cast< int >( 1000 * sin( ( static_cast< int >( _dInputJoystate.POV[ 0 ] ) - 9000 ) * DX_PI / 18000 ) );
	}
}

void Input::LoadConfig( void ){
	std::ifstream ifs( "key.ini", std::ios::in );
	std::string str;
	if( !ifs ){
		LoadDefaultConfig();
		SaveDefaultConfig();
	}
	else{
		ifs.ignore( 100, '=' );
		ifs >> _config._attackUp;
		ifs.ignore( 100, '=' );
		ifs >> _config._attackDown;
		ifs.ignore( 100, '=' );
		ifs >> _config._attackLeft;
		ifs.ignore( 100, '=' );
		ifs >> _config._attackRight;
		ifs.ignore( 100, '=' );
		ifs >> _config._pause;
		
		std::string str;

		ifs.ignore( 100, '=' );
		std::getline( ifs, str );
		if( str == "Rx" || str == "rx" || str == "RX" ){
			_config._attackAnalog[ 0 ] = 1;
		}
		else if( str == "Ry" || str == "ry" || str == "RY" ){
			_config._attackAnalog[ 0 ] = 2;
		}
		else if( str == "Rz" || str == "rz" || str == "RZ" ){
			_config._attackAnalog[ 0 ] = 3;
		}
		else{
			_config._attackAnalog[ 0 ] = 0;
		}
		
		ifs.ignore( 100, '=' );
		std::getline( ifs, str );
		if( str == "Rx" || str == "rx" || str == "RX" ){
			_config._attackAnalog[ 1 ] = 1;
		}
		else if( str == "Ry" || str == "ry" || str == "RY" ){
			_config._attackAnalog[ 1 ] = 2;
		}
		else if( str == "Rz" || str == "rz" || str == "RZ" ){
			_config._attackAnalog[ 1 ] = 3;
		}
		else{
			_config._attackAnalog[ 1 ] = 0;
		}

		ifs.ignore( 100, '=' );
		std::getline( ifs, str );
		if( str == "false" || str == "FALSE" || str == "False" ){
			_config._useDirectInputFlag = false;
		}
		else{
			_config._useDirectInputFlag = true;
		}


		// ͈͊Ȍꍇ̓ftHg̃RtBO[h
		if( _config._attackUp < 1 || 32 < _config._attackUp ||
			_config._attackDown < 1 || 32 < _config._attackDown ||
			_config._attackLeft < 1 || 32 < _config._attackLeft ||
			_config._attackRight < 1 || 32 < _config._attackRight ||
			_config._pause < 1 || 32 < _config._pause )
			LoadDefaultConfig();
	}
	return;
}
void Input::LoadDefaultConfig( void ){
	_config._attackUp = 1;
	_config._attackDown = 3;
	_config._attackLeft = 4;
	_config._attackRight = 2;
	_config._pause = 10;
	// _config._attackAnalog[ 0 ] = 0;
	// _config._attackAnalog[ 1 ] = 1;
	_config._attackAnalog[ 0 ] = 3;
	_config._attackAnalog[ 1 ] = 0;
	_config._useDirectInputFlag = true;
}
void Input::SaveDefaultConfig( void ){
	std::ofstream ofs( "key.ini" );
	ofs << "ATTACK_UP=1" << std::endl;
	ofs << "ATTACK_DOWN=3" << std::endl;
	ofs << "ATTACK_LEFT=4" << std::endl;
	ofs << "ATTACK_RIGHT=2" << std::endl;
	ofs << "PAUSE=10" << std::endl;
	ofs << "ATTACK_X=RZ" << std::endl;
	ofs << "ATTACK_Y=Z" << std::endl;
	ofs << "USE_DIRECT_INPUT_FLAG=TRUE" << std::endl;
}