#include "Player.h"
#include "DxLib.h"
#include "Input.h"
#include <math.h>
#include "Shot.h"
#include <sstream>
#include "Effect.h"

Player* Player::_player;
struct Player::Sound Player::_snd;
int Player::_model[];
int Player::_img;
extern int g_gameover;

Player::Player(){
	_x = 320;
	_y = 240;
	_th[ 0 ] = -DX_PI_F / 2;
	_th[ 1 ] = -DX_PI_F / 2;
	_count = 0;
	_player = this;
	_life = 3;
	_invincible = 0;
	_shotPower = 1;
}
int Player::DrawLife( void ){
	std::ostringstream os;
	int life = _life < 0 ? 0 : _life;
	os << "LIFE: " << life;
	return DrawFormatString( 0, 0, GetColor( 255, 255, 255 ), os.str().c_str() );
}
int Player::Draw( void ){
	_count++;
	int r = 0;
	DrawLife();
	if ( ( _invincible % 2 ) == 0 ) {
		int x = 0, y = 0;
		if( _life == -1 ){
			x = GetRand( _count ) - _count / 2;
			y = GetRand( _count ) - _count / 2;
			if( _count % 8 == 0 ){
				PlaySoundMem( _snd._damage, DX_PLAYTYPE_BACK );
				new Effect( _x + x, _y + y, 1.0 );
			}
			if( _count == 200 ){
				_life = -2;
				PlaySoundMem( _snd._death, DX_PLAYTYPE_BACK );
				_count = 0;
			}
		}
		if( _life == -2 ){
			if( _count == 200 ){
				_life = -3;
			}
			return DrawRotaGraph( 320, 240, 2.0, 0, g_gameover, TRUE );
		}
		r |= DrawRotaGraph( static_cast< int >( _x ) + x, static_cast< int >( _y ) + y, 1.0 + 0.05 + sin( static_cast< double >( _count ) / 10 ) * 0.1, 0, _img, TRUE );
		MV1SetPosition( _model[ 0 ], VGet( _x + x, 480 - _y - y, 0 ) );
		MV1SetPosition( _model[ 1 ], VGet( _x + x, 480 - _y - y, 0 ) );
		MV1SetRotationXYZ( _model[ 0 ], VGet( static_cast< float >( _count ) / 30, 0, -_th[ 0 ] ) );
		MV1SetRotationXYZ( _model[ 1 ], VGet( -static_cast< float >( _count ) / 30, 0, -_th[ 1 ] ) );
		r |= MV1DrawModel( _model[ 0 ] );
		r |= MV1DrawModel( _model[ 1 ] );
#ifdef _DEBUG
		r |= DrawCircle( static_cast< int >( _x ), static_cast< int >( _y ),
			GetRadius(), GetColor( 0, 255, 0 ), false );
#endif
	}
	if ( _invincible ) {
		_invincible--;
	}
	return r;
}

void Player::Move( void ){
	if( _life < 0 ) return;
	double x[ 2 ], y[ 2 ];
	x[ 0 ] = Input::GetAnalog( 0 );
	y[ 0 ] = Input::GetAnalog( 1 );
	x[ 1 ] = Input::GetAnalog( 2 );
	y[ 1 ] = Input::GetAnalog( 3 );
	
	if( x[ 1 ] != 0 || y[ 1 ] != 0 ){
		_th[ 0 ] = static_cast< float >( atan2( y[ 1 ], x[ 1 ] ) );
	}
	if( x[ 0 ] != 0 || y[ 0 ] != 0 ){
		_th[ 1 ] = static_cast< float >( atan2( y[ 0 ], x[ 0 ] ) );
	}

	_x += static_cast< float >( Input::GetAnalog( 0 ) ) / 150;
	_y += static_cast< float >( Input::GetAnalog( 1 ) ) / 150;
	
	if( _x < 32 ) _x = 32;
	if( 640 - 32 < _x ) _x = 640 - 32;
	if( _y < 32 ) _y = 32;
	if( 480 - 32 < _y ) _y = 480 - 32;

	double vx = Input::GetAnalog( 2 );
	double vy = Input::GetAnalog( 3 );
	
	if ( ( vx != 0 || vy != 0 ) && _count % 3 == 0 ) {
		static const double ELEVATION = 0.25;
		static const double SPACE = 16;
		PlaySoundMem( _snd._shot, DX_PLAYTYPE_BACK );
		double direction = atan2( vy, vx );
		switch( _shotPower ){
		case 1:
			new Shot( _x, _y, direction );
			break;
		case 2:
			new Shot(
				_x + SPACE * cos( direction + DX_PI / 2 ),
				_y + SPACE * sin( direction + DX_PI / 2 ),
				direction );
			new Shot(
				_x + SPACE * cos( direction - DX_PI / 2 ),
				_y + SPACE * sin( direction - DX_PI / 2 ),
				direction );
			break;
		case 3:
			new Shot( _x, _y, direction );
			new Shot( _x, _y, direction + ELEVATION );
			new Shot( _x, _y, direction - ELEVATION );
			break;
		}
	}
}
int Player::LoadSnd( int volume ){
	if( volume > 255 ) volume = 255;
	_snd._shot		= LoadSoundMem( "snd\\shot.wav"		);
	_snd._damage	= LoadSoundMem( "snd\\damage.wav"	);
	_snd._item		= LoadSoundMem( "snd\\item.wav"		);
	_snd._death		= LoadSoundMem( "snd\\death.wav"	);
	ChangeVolumeSoundMem( volume, _snd._damage	);
	ChangeVolumeSoundMem( volume, _snd._shot	);
	ChangeVolumeSoundMem( volume, _snd._item	);
	ChangeVolumeSoundMem( volume, _snd._death	);
	return 0;
}
void Player::ShotPowerUp( void ){
	static const int SHOT_POWER_MAX = 3;
	PlaySoundMem( _snd._item, DX_PLAYTYPE_BACK );
	if( _player->_shotPower < SHOT_POWER_MAX )
		_player->_shotPower++;
}
int Player::LoadModel( void ){
	_img = LoadGraph( "img\\player.png" );
	_model[ 0 ] = MV1LoadModel( "model\\player.x" );
	_model[ 1 ] = MV1DuplicateModel( _model[ 0 ] ) ;
	MV1SetWireFrameDrawFlag( _model[ 0 ],TRUE );
	MV1SetWireFrameDrawFlag( _model[ 1 ],TRUE );
	MV1SetUseZBuffer( _model[ 0 ], FALSE ) ;
	MV1SetUseZBuffer( _model[ 1 ], FALSE ) ;
	MV1SetScale( _model[ 1 ], VGet( 0.5, 0.5f, 0.5 ) );
	return 0;
}
void Player::Hit( void ){
	if( !( _player->_invincible ) && _player->_life >= 0 ){
		PlaySoundMem( _snd._damage, DX_PLAYTYPE_BACK );
		_player->_life--;
		if( _player->_life == -1 ){
			_player->_count = 0;
		}
		else {
			_player->_invincible = 100;
			_player->_shotPower = 1;
		}
	}
}